package e4.performance.sample.handler;

import org.eclipse.gef.geometry.planar.Dimension;
import org.eclipse.gef.mvc.fx.handlers.AbstractHandler;
import org.eclipse.gef.mvc.fx.handlers.IOnDragHandler;
import org.eclipse.gef.mvc.fx.handlers.IOnStrokeHandler;
import org.eclipse.gef.mvc.fx.policies.TransformPolicy;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import e4.performance.sample.part.CanPart;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.MouseEvent;

public class TranslateViewOnDragHandler extends AbstractHandler implements IOnDragHandler, IOnStrokeHandler
{
  protected Logger logger = LoggerFactory.getLogger(getClass());

  private boolean isControlDown = false;
  private boolean invalidPanning = false;
  private TransformPolicy transformPolicy = null;
  private int postTransformSize = 0;

  @Override
  public void abortDrag()
  {
    if (invalidPanning)
    {
      return;
    }
    rollback(transformPolicy);
    logger.info(getHost().toString());
    restoreRefreshVisuals(getHost());
    transformPolicy = null;
  }

  @Override
  public void drag(MouseEvent e, Dimension delta)
  {
    if (invalidPanning)
    {
      return;
    }
    transformPolicy.setPostTranslate(postTransformSize, delta.width, delta.height);
  }

  @Override
  public void endDrag(MouseEvent e, Dimension delta)
  {
    if (invalidPanning)
    {
      return;
    }
    commit(transformPolicy);
    restoreRefreshVisuals(getRoot());
    transformPolicy = null;
  }

  @Override
  public void hideIndicationCursor()
  {
  }

  @Override
  public boolean showIndicationCursor(KeyEvent event)
  {
    return false;
  }

  @Override
  public boolean showIndicationCursor(MouseEvent event)
  {
    return false;
  }

  @Override
  public void startDrag(MouseEvent e)
  {
    invalidPanning = !isControlDown;
    if (invalidPanning)
    {
      return;
    }
    CanPart canPart = getRoot();
    transformPolicy = canPart.getAdapter(TransformPolicy.class);

    storeAndDisableRefreshVisuals(canPart);
    init(transformPolicy);
    postTransformSize = transformPolicy.createPostTransform();
  }

  public CanPart getRoot()
  {
    return getHost().getViewer().getContentPartMap().values().stream().filter(CanPart.class::isInstance).map(CanPart.class::cast).findFirst().get();
  }

  @Override
  public void finalRelease(KeyEvent event)
  {
    if (event.getCode().equals(KeyCode.CONTROL))
    {
      isControlDown = false;
    }
  }

  @Override
  public void initialPress(KeyEvent event)
  {
    if (event.getCode().equals(KeyCode.CONTROL))
    {
      isControlDown = true;
    }
  }

  @Override
  public void press(KeyEvent event)
  {
    if (event.getCode().equals(KeyCode.CONTROL))
    {
      isControlDown = true;
    }
  }

  @Override
  public void release(KeyEvent event)
  {
    if (event.getCode().equals(KeyCode.CONTROL))
    {
      isControlDown = false;
    }
  }

  @Override
  public void abortPress()
  {
  }
}
